/*
 * JBoss, Home of Professional Open Source
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.cache.options.cachemodelocal;

import org.jboss.cache.config.Configuration;
import org.jboss.cache.util.internals.ReplicationListener;
import org.jboss.cache.commands.ReplicableCommand;
import org.jboss.cache.commands.legacy.write.PessPutDataMapCommand;
import org.jboss.cache.commands.legacy.write.PessPutKeyValueCommand;
import org.jboss.cache.commands.legacy.write.PessRemoveKeyCommand;
import org.jboss.cache.commands.legacy.write.PessRemoveNodeCommand;
import org.jboss.cache.commands.write.PutDataMapCommand;
import org.jboss.cache.commands.write.PutKeyValueCommand;
import org.jboss.cache.commands.write.RemoveKeyCommand;
import org.jboss.cache.commands.write.RemoveNodeCommand;
import org.jboss.cache.CacheSPI;
import org.jboss.cache.Cache;
import org.testng.annotations.Test;

import java.util.Map;
import java.util.HashMap;

@Test(groups = {"functional", "jgroups"}, testName = "options.cachemodelocal.AsyncReplPessLocksTest")
public class AsyncReplPessLocksTest extends CacheModeLocalTestBase
{

   ReplicationListener current;

   Map<Cache, ReplicationListener> cache2Listener = new HashMap<Cache, ReplicationListener>(2);

   public AsyncReplPessLocksTest()
   {
      cacheMode = Configuration.CacheMode.REPL_ASYNC;
      nodeLockingScheme = "PESSIMISTIC";
   }

   protected void registerReplicationCommand(CacheSPI<String, String> cache, Class<? extends ReplicableCommand> what)
   {
      if (what == PutDataMapCommand.class) what = PessPutDataMapCommand.class;
      if (what == PutKeyValueCommand.class) what = PessPutKeyValueCommand.class;
      if (what == RemoveKeyCommand.class) what = PessRemoveKeyCommand.class;
      if (what == RemoveNodeCommand.class) what = PessRemoveNodeCommand.class;
      getCacheListener(cache).expect(what);
   }

   protected void verifyReplication()
   {
      current.waitForReplicationToOccur(2000);
   }

   public ReplicationListener getCacheListener(Cache cache)
   {
      current = cache2Listener.get(cache);
      if (current == null)
      {
         current = new ReplicationListener(cache);
         cache2Listener.put(cache, current);
      }
      return current;
   }
}
