package org.jboss.cache.util;

import java.util.Properties;
import java.util.regex.Pattern;
import java.util.regex.Matcher;
import java.io.IOException;

/**
 * @author Mircea.Markus@jboss.com
 */
public class TestDbPropertiesFactory
{
   private static final Properties realProps = new Properties();

   private static int userIndex;

   static {
      //so that all individual databases are created here
      System.setProperty("derby.system.home","derby");
      try
      {
         realProps.load(TestDbPropertiesFactory.class.getClassLoader().getResourceAsStream("cache-jdbc.properties"));
      } catch (IOException e)
      {
         e.printStackTrace();
         throw new Error("Missing property file: cache-jdbc.properties",e);
      }
   }

   public static Properties getTestDbProperties()
   {
       synchronized (realProps)
       {
          return returnBasedOnDifferentInstance();
       }
   }

   private static Properties returnBasedOnDifferentInstance()
   {
      Properties toReturn = (Properties) realProps.clone();
      String jdbcUrl = toReturn.getProperty("cache.jdbc.url");
      Pattern pattern = Pattern.compile("jbossdb");
      Matcher matcher = pattern.matcher(jdbcUrl);
      boolean found = matcher.find();
      assert found;
      String newJdbcUrl = matcher.replaceFirst("jbossdb" + userIndex++);
      System.out.println("newJdbcUrl = " + newJdbcUrl);
      toReturn.put("cache.jdbc.url", newJdbcUrl);
      return toReturn;
   }

   private static Properties returnBasedOnDifferentTables()
   {
      Properties toReturn = (Properties) realProps.clone();
      String tableName = toReturn.getProperty("cache.jdbc.table.name");
      int currentIndex = userIndex++;
      toReturn.setProperty("cache.jdbc.table.name", tableName + currentIndex);
      String pk = toReturn.getProperty("cache.jdbc.table.primarykey");
      toReturn.setProperty("cache.jdbc.table.primarykey", pk + currentIndex);
      return toReturn;
   }
}
